==> lgob-9.04.tar.bz2: lgob-9.04/gobject/src/interface.c <==
/*
	This file is part of lgob.

	lgob is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lgob is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lgob.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2009 Lucas Hermann Negri
*/

#include "interface.h"

static void priv_register(lua_State* L, const char* name, lua_CFunction func)
{
	lua_pushstring(L, name);
	lua_pushcfunction(L, func);
	lua_rawset(L, LUA_REGISTRYINDEX);
};

/* GObject */

static const struct luaL_reg gobject [] =
{
	{NULL, NULL}
};

static const struct luaL_reg object [] =
{	
	{"set", lgob_object_set},
	{"get", lgob_object_get},
	{"connect", lgob_object_connect},
	{"disconnect", lgob_object_disconnect},
	{"block", lgob_object_block},
	{"unblock", lgob_object_unblock},
	{"ref", lgob_object_ref},
	{"unref", lgob_object_unref},
	{"notify", lgob_object_notify},
	{"thaw_notify", lgob_object_thaw_notify},
	{"freeze_notify", lgob_object_freeze_notify},
	{"cast", lgob_object_cast},
	{NULL, NULL}
};

static const struct luaL_reg type [] =
{
	{"from_name", lgob_type_from_name},
	{"from_instance", lgob_type_from_instance},
	{"name_from_type", lgob_type_name_from_type},
	{"parent", lgob_type_parent},
	{"depth", lgob_type_depth},
	{"is_a", lgob_type_is_a},
	{NULL, NULL}
};

/* GLib */

static const struct luaL_reg glib [] =
{
	{"version", lgob_version},
	{"convert", lgob_convert},
	{"get_charset", lgob_get_charset},
	{"thread_init", lgob_thread_init},
	{NULL, NULL}
};

static const struct luaL_reg timeout [] =
{
	{"add", lgob_timeout_add},
	{"iddle_add", lgob_timeout_idle_add},
	{"remove", lgob_timeout_remove},
	{NULL, NULL}
};

static const struct luaL_reg mainloop [] =
{
	{"new", lgob_main_loop_new},
	{"iterate", lgob_main_loop_iterate},
	{"pending", lgob_main_loop_pending},
	{"quit", lgob_main_loop_quit},
	{"run", lgob_main_loop_run},
	{NULL, NULL}
};

static const struct luaL_reg environment [] =
{
#if GLIB_MINOR_VERSION >= 8
	{"get_host_name", lgob_environment_get_host_name},
#endif
	{"get_current_dir", lgob_environment_get_current_dir},
	{"get_home_dir", lgob_environment_get_home_dir},
	{"get_real_name", lgob_environment_get_real_name},
	{"get_tmp_dir", lgob_environment_get_tmp_dir},
	{"get_user_cache_dir", lgob_environment_get_user_cache_dir},
	{"get_user_config_dir", lgob_environment_get_user_config_dir},
	{"get_user_data_dir", lgob_environment_get_user_data_dir},
	{"get_user_name", lgob_environment_get_user_name},
	{NULL, NULL}
};

static const struct luaL_reg path [] =
{
	{"find_program", lgob_path_find_program},
	{"get_basename", lgob_path_get_basename},
	{"get_dirname", lgob_path_get_dirname},
	{"is_absolute", lgob_path_is_absolute},
	{"skip_root", lgob_path_skip_root},
	{NULL, NULL}
};

static const struct luaL_reg rand [] =
{
	{"boolean", lgob_rand_boolean},
	{"double", lgob_rand_double},
	{"int", lgob_rand_int},
	{"new", lgob_rand_new},
	{"set_seed", lgob_rand_set_seed},
	{NULL, NULL}
};

static const struct luaL_reg timer [] =
{
	{"continue", lgob_timer_continue},
	{"elapsed", lgob_timer_elapsed},
	{"new", lgob_timer_new},
	{"start", lgob_timer_start},
	{"stop", lgob_timer_stop},
	{NULL, NULL}
};

static const struct luaL_reg utf8 [] =
{
	{"casefold", lgob_utf8_casefold},
	{"collate", lgob_utf8_collate},
	{"collate_key", lgob_utf8_collate_key},
	{"from_filename", lgob_utf8_from_filename},
	{"normalize", lgob_utf8_normalize},
	{"strdown", lgob_utf8_strdown},
	{"strlen", lgob_utf8_strlen},
	{"strup", lgob_utf8_strup},
	{"to_filename", lgob_utf8_to_filename},
	{"validate", lgob_utf8_validate},
	{"get_charset", lgob_get_charset},
	{NULL, NULL}
};

static const struct luaL_reg timeval [] =
{
	{"add", lgob_time_val_add},
	{"new", lgob_time_val_new},
#if GLIB_MINOR_VERSION >= 10
	{"from_iso8601", lgob_time_val_from_iso8601},
	{"to_iso8601", lgob_time_val_to_iso8601},
	{NULL, NULL}
#endif
};

#if GLIB_MINOR_VERSION >= 12

static const struct luaL_reg base64 [] =
{
	{"decode", lgob_base64_decode},
	{"encode", lgob_base64_encode},
	{NULL, NULL}
};

#endif

#if GLIB_MINOR_VERSION >= 16

static const struct luaL_reg checksum [] =
{
	{"get_md5", lgob_checksum_get_md5},
	{"get_sha1", lgob_checksum_get_sha1},
	{"get_sha256", lgob_checksum_get_sha256},
	{NULL, NULL}
};

#endif

static const struct luaL_reg date [] =
{
	{"compare", lgob_date_compare},
	{"get_day", lgob_date_get_day},
	{"get_month", lgob_date_get_month},
	{"get_year", lgob_date_get_year},
	{"is_leap_year", lgob_date_is_leap_year},
	{"new", lgob_date_new},
	{"set_dmy", lgob_date_set_dmy},
	{"set_julian", lgob_date_set_julian},
	{"set_parse", lgob_date_set_parse},
#if GLIB_MINOR_VERSION >= 10	
	{"set_time_t", lgob_date_set_time_t},
#endif	
	{NULL, NULL}
};

static int index_closure(lua_State* L)
{	
	/* Organize the stack to be: 1 - udata, 2 - property, 3 - value (if present) */
	size_t len;
	const char* p = lua_tolstring(L, lua_upvalueindex(1), &len);
	
	#ifdef IDEBUG
	g_debug("Calling index_closure for key %s!", p);
	#endif
	
	lua_pushvalue(L, 2);
	
	if(p[1] == 'e' && p[2] == 't' && len >= 5)
	{
		lua_pushstring(L, &p[4]);
		lua_replace(L, 2);
		
		if(p[0] == 'g') /* getter */
		{
			lgob_object_get(L);
			return 1;
		}
		else
			if(p[0] == 's') /* setter */
			{
				lgob_object_set(L);
				return 0;
			}
	}
	
	luaL_error(L, "Invalid function (%s)", p);
	return 0;
}

static int __index(lua_State* L)
{
	lua_pushvalue(L, 2);
	lua_pushcclosure(L, index_closure, 1);
	
	return 1;
}

int luaopen_lgob_gobject(lua_State *L)
{
	/* Initialization */
	g_type_init();
	luaL_loadstring(L, "require('lgob.common')"); lua_call(L, 0, 0);
	
	/* Register the classes */
	luaL_register(L, LIB_NAME, gobject);
	int top = lua_gettop(L);
	
	lua_pushliteral(L, "lgobObject");
	luaL_register(L, "gobject.Object", object);
	
	/* Set the object metatable */
	lua_newtable(L);
	lua_pushliteral(L, "__index");
	lua_pushcfunction(L, __index);
	lua_rawset(L, -3);
	
	lua_setmetatable(L, -2);
	
	lua_rawset(L, LUA_REGISTRYINDEX);
	luaL_register(L, "gobject.Type", type);
	
	/* Export some internal functions to other bindings */
	priv_register(L, "lgobObjectNew", lgob_object_new); 
	priv_register(L, "lgobStructNew", lgob_struct_new); 
	priv_register(L, "lgobBoxedNew", lgob_boxed_new); 
	priv_register(L, "lgobGc", lgob_object_gc); 
	priv_register(L, "lgobValuePush", lgob_value_push); 
	priv_register(L, "lgobValueSet", lgob_value_set); 
	priv_register(L, "lgobStructGc", lgob_struct_gc); 
	priv_register(L, "lgobStructTostring", lgob_struct_to_string); 
	priv_register(L, "lgobTostring", lgob_object_to_string); 
	priv_register(L, "lgobEq", lgob_object_eq);
	priv_register(L, "lgobClosureNew", lgob_closure_new);
	priv_register(L, "lgobGstrv2table", lgob_gstrv2table);
	priv_register(L, "lgobTable2gstrv", lgob_table2gstrv);
	priv_register(L, "lgobGSList2table", lgob_gslist2table);
	priv_register(L, "lgobGList2table", lgob_glist2table);
	
	/* Create a table to store class name exceptions */
	lua_newtable(L);
	lua_setfield(L, LUA_REGISTRYINDEX, "lgobSpecial");
	
	/* Create a table to store prefix */
	lua_newtable(L);
	lua_pushliteral(L, "G"); lua_pushliteral(L, "lgob"); lua_rawset(L, -3);
	lua_setfield(L, LUA_REGISTRYINDEX, "lgobPrefix");
	
	/* Register the GLib binding */
	luaL_register(L, "glib", glib);
	
	/* G_PRIORITY define */
	lua_pushliteral(L, "PRIORITY_HIGH"); lua_pushinteger(L, G_PRIORITY_HIGH); lua_rawset(L, -3);
	lua_pushliteral(L, "PRIORITY_DEFAULT"); lua_pushinteger(L, G_PRIORITY_DEFAULT); lua_rawset(L, -3);
	lua_pushliteral(L, "PRIORITY_HIGH_IDLE"); lua_pushinteger(L, G_PRIORITY_HIGH_IDLE); lua_rawset(L, -3);
	lua_pushliteral(L, "PRIORITY_DEFAULT_IDLE"); lua_pushinteger(L, G_PRIORITY_DEFAULT_IDLE); lua_rawset(L, -3);
	lua_pushliteral(L, "PRIORITY_LOW"); lua_pushinteger(L, G_PRIORITY_LOW); lua_rawset(L, -3);
	
	/* GNormalizeMode enum */
	lua_pushliteral(L, "NORMALIZE_DEFAULT"); lua_pushnumber(L, G_NORMALIZE_DEFAULT); lua_rawset(L, -3);
	lua_pushliteral(L, "NORMALIZE_NFD"); lua_pushnumber(L, G_NORMALIZE_NFD); lua_rawset(L, -3);
	lua_pushliteral(L, "NORMALIZE_DEFAULT_COMPOSE"); lua_pushnumber(L, G_NORMALIZE_DEFAULT_COMPOSE); lua_rawset(L, -3);
	lua_pushliteral(L, "NORMALIZE_NFC"); lua_pushnumber(L, G_NORMALIZE_NFC); lua_rawset(L, -3);
	lua_pushliteral(L, "NORMALIZE_ALL"); lua_pushnumber(L, G_NORMALIZE_ALL); lua_rawset(L, -3);
	lua_pushliteral(L, "NORMALIZE_NFKD"); lua_pushnumber(L, G_NORMALIZE_NFKD); lua_rawset(L, -3);
	lua_pushliteral(L, "NORMALIZE_ALL_COMPOSE"); lua_pushnumber(L, G_NORMALIZE_ALL_COMPOSE); lua_rawset(L, -3);
	lua_pushliteral(L, "NORMALIZE_NFKC"); lua_pushnumber(L, G_NORMALIZE_NFKC); lua_rawset(L, -3);
	
	luaL_register(L, "glib.Timeout", timeout);
	luaL_register(L, "glib.MainLoop", mainloop);
	luaL_register(L, "glib.Environment", environment);
	luaL_register(L, "glib.Path", path);
	luaL_register(L, "glib.Date", date);
	luaL_register(L, "glib.Rand", rand);
	luaL_register(L, "glib.TimeVal", timeval);
	luaL_register(L, "glib.Timer", timer);
	luaL_register(L, "glib.UTF8", utf8);
	
#if GLIB_MINOR_VERSION >= 12
	luaL_register(L, "glib.Base64", base64);
#endif

#if GLIB_MINOR_VERSION >= 16
	luaL_register(L, "glib.Checksum", checksum);
#endif
	
	if(luaL_newmetatable(L, "glibMainLoopMT"))
	{
		lua_getfield(L, LUA_REGISTRYINDEX, "lgobRegisterSpecial");
		lua_pushvalue(L, -2);
		lua_pushliteral(L, "glib");
		lua_pushliteral(L, "MainLoop");
		lua_pushcfunction(L, lgob_main_loop_free);
		lua_call(L, 4, 0);
	}
	
	if(luaL_newmetatable(L, "glibDateMT"))
	{
		lua_getfield(L, LUA_REGISTRYINDEX, "lgobRegisterSpecial");
		lua_pushvalue(L, -2);
		lua_pushliteral(L, "glib");
		lua_pushliteral(L, "Date");
		lua_pushcfunction(L, lgob_date_free);
		lua_call(L, 4, 0);
	}
	
	if(luaL_newmetatable(L, "glibRandMT"))
	{
		lua_getfield(L, LUA_REGISTRYINDEX, "lgobRegisterSpecial");
		lua_pushvalue(L, -2);
		lua_pushliteral(L, "glib");
		lua_pushliteral(L, "Rand");
		lua_pushcfunction(L, lgob_rand_free);
		lua_call(L, 4, 0);
	}
	
	if(luaL_newmetatable(L, "glibTimeValMT"))
	{
		lua_getfield(L, LUA_REGISTRYINDEX, "lgobRegisterSpecial");
		lua_pushvalue(L, -2);
		lua_pushliteral(L, "glib");
		lua_pushliteral(L, "TimeVal");
		lua_pushcfunction(L, lgob_time_val_free);
		lua_call(L, 4, 0);
	}
	
	if(luaL_newmetatable(L, "glibTimerMT"))
	{
		lua_getfield(L, LUA_REGISTRYINDEX, "lgobRegisterSpecial");
		lua_pushvalue(L, -2);
		lua_pushliteral(L, "glib");
		lua_pushliteral(L, "Timer");
		lua_pushcfunction(L, lgob_timer_free);
		lua_call(L, 4, 0);
	}
	
	/* lgob table must be in the top */
	lua_settop(L, top);

	return 1;
}

==> lgob-9.06.1.tar.bz2: lgob-9.06.1/gobject/src/interface.c <==
/*
	This file is part of lgob.

	lgob is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lgob is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lgob.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2009 Lucas Hermann Negri
*/

#include "interface.h"

static void priv_register(lua_State* L, const char* name, lua_CFunction func)
{
	lua_pushstring(L, name);
	lua_pushcfunction(L, func);
	lua_rawset(L, LUA_REGISTRYINDEX);
};

/* GObject */

static const struct luaL_reg gobject [] =
{
	{NULL, NULL}
};

static const struct luaL_reg object [] =
{	
	{"set", lgob_object_set},
	{"get", lgob_object_get},
	{"connect", lgob_object_connect},
	{"disconnect", lgob_object_disconnect},
	{"block", lgob_object_block},
	{"unblock", lgob_object_unblock},
	{"ref", lgob_object_ref},
	{"unref", lgob_object_unref},
	{"notify", lgob_object_notify},
	{"thaw_notify", lgob_object_thaw_notify},
	{"freeze_notify", lgob_object_freeze_notify},
	{"cast", lgob_object_cast},
	{NULL, NULL}
};

static const struct luaL_reg type [] =
{
	{"from_name", lgob_type_from_name},
	{"from_instance", lgob_type_from_instance},
	{"name_from_type", lgob_type_name_from_type},
	{"parent", lgob_type_parent},
	{"depth", lgob_type_depth},
	{"is_a", lgob_type_is_a},
	{NULL, NULL}
};

/* GLib */

static void priv_log_handler(const gchar *log_domain, GLogLevelFlags log_level,
	const gchar *message, lua_State* L)
{
	/* Show the line of the Lua program */
	luaL_where(L, 1);
	if(lua_objlen(L, -1) == 0) luaL_where(L, 1);
	if(lua_objlen(L, -1) == 0) lua_pushstring(L, "Mystical source! ");
	
	/* Build the message */
	lua_pushstring(L, log_domain);
	lua_pushstring(L, " - ");
	lua_pushstring(L, message);
	lua_pushstring(L, "\n");
	
	/* Log it */
	lua_concat(L, 5);
	g_printerr("%s", lua_tostring(L, -1));
	lua_pop(L, 1);
}

static void priv_handle_log(lua_State* L, const char* domain)
{
	g_log_set_handler(domain, G_LOG_LEVEL_WARNING | G_LOG_LEVEL_CRITICAL | G_LOG_FLAG_RECURSION, (GLogFunc)priv_log_handler, L);
}

static int lgob_handle_log(lua_State* L)
{
	priv_handle_log(L, luaL_checkstring(L, 1));
	return 0;
}

static const struct luaL_reg glib [] =
{
	{"version", lgob_version},
	{"convert", lgob_convert},
	{"get_charset", lgob_get_charset},
	{"thread_init", lgob_thread_init},
	{"handle_log", lgob_handle_log},
	{NULL, NULL}
};

static const struct luaL_reg timeout [] =
{
	{"add", lgob_timeout_add},
	{"iddle_add", lgob_timeout_idle_add},
	{"remove", lgob_timeout_remove},
	{NULL, NULL}
};

static const struct luaL_reg mainloop [] =
{
	{"new", lgob_main_loop_new},
	{"iterate", lgob_main_loop_iterate},
	{"pending", lgob_main_loop_pending},
	{"quit", lgob_main_loop_quit},
	{"run", lgob_main_loop_run},
	{NULL, NULL}
};

static const struct luaL_reg environment [] =
{
#if GLIB_MINOR_VERSION >= 8
	{"get_host_name", lgob_environment_get_host_name},
#endif
	{"get_current_dir", lgob_environment_get_current_dir},
	{"get_home_dir", lgob_environment_get_home_dir},
	{"get_real_name", lgob_environment_get_real_name},
	{"get_tmp_dir", lgob_environment_get_tmp_dir},
	{"get_user_cache_dir", lgob_environment_get_user_cache_dir},
	{"get_user_config_dir", lgob_environment_get_user_config_dir},
	{"get_user_data_dir", lgob_environment_get_user_data_dir},
	{"get_user_name", lgob_environment_get_user_name},
	{NULL, NULL}
};

static const struct luaL_reg path [] =
{
	{"find_program", lgob_path_find_program},
	{"get_basename", lgob_path_get_basename},
	{"get_dirname", lgob_path_get_dirname},
	{"is_absolute", lgob_path_is_absolute},
	{"skip_root", lgob_path_skip_root},
	{NULL, NULL}
};

static const struct luaL_reg rand [] =
{
	{"boolean", lgob_rand_boolean},
	{"double", lgob_rand_double},
	{"int", lgob_rand_int},
	{"new", lgob_rand_new},
	{"set_seed", lgob_rand_set_seed},
	{NULL, NULL}
};

static const struct luaL_reg timer [] =
{
	{"continue", lgob_timer_continue},
	{"elapsed", lgob_timer_elapsed},
	{"new", lgob_timer_new},
	{"start", lgob_timer_start},
	{"stop", lgob_timer_stop},
	{NULL, NULL}
};

static const struct luaL_reg utf8 [] =
{
	{"casefold", lgob_utf8_casefold},
	{"collate", lgob_utf8_collate},
	{"collate_key", lgob_utf8_collate_key},
	{"from_filename", lgob_utf8_from_filename},
	{"normalize", lgob_utf8_normalize},
	{"strdown", lgob_utf8_strdown},
	{"strlen", lgob_utf8_strlen},
	{"strup", lgob_utf8_strup},
	{"to_filename", lgob_utf8_to_filename},
	{"validate", lgob_utf8_validate},
	{"get_charset", lgob_get_charset},
	{NULL, NULL}
};

static const struct luaL_reg timeval [] =
{
	{"add", lgob_time_val_add},
	{"new", lgob_time_val_new},
#if GLIB_MINOR_VERSION >= 10
	{"from_iso8601", lgob_time_val_from_iso8601},
	{"to_iso8601", lgob_time_val_to_iso8601},
	{NULL, NULL}
#endif
};

#if GLIB_MINOR_VERSION >= 12

static const struct luaL_reg base64 [] =
{
	{"decode", lgob_base64_decode},
	{"encode", lgob_base64_encode},
	{NULL, NULL}
};

#endif

#if GLIB_MINOR_VERSION >= 16

static const struct luaL_reg checksum [] =
{
	{"get_md5", lgob_checksum_get_md5},
	{"get_sha1", lgob_checksum_get_sha1},
	{"get_sha256", lgob_checksum_get_sha256},
	{NULL, NULL}
};

#endif

static const struct luaL_reg date [] =
{
	{"compare", lgob_date_compare},
	{"get_day", lgob_date_get_day},
	{"get_month", lgob_date_get_month},
	{"get_year", lgob_date_get_year},
	{"is_leap_year", lgob_date_is_leap_year},
	{"new", lgob_date_new},
	{"set_dmy", lgob_date_set_dmy},
	{"set_julian", lgob_date_set_julian},
	{"set_parse", lgob_date_set_parse},
#if GLIB_MINOR_VERSION >= 10	
	{"set_time_t", lgob_date_set_time_t},
#endif	
	{NULL, NULL}
};

static int index_closure(lua_State* L)
{	
	/* Organize the stack to be: 1 - udata, 2 - property, 3 - value (if present) */
	size_t len;
	const char* p = lua_tolstring(L, lua_upvalueindex(1), &len);
	
	#ifdef IDEBUG
	g_debug("Calling index_closure for key %s!", p);
	#endif
	
	lua_pushvalue(L, 2);
	
	if(p[1] == 'e' && p[2] == 't' && len >= 5)
	{
		lua_pushstring(L, &p[4]);
		lua_replace(L, 2);
		
		if(p[0] == 'g') /* getter */
		{
			lgob_object_get(L);
			return 1;
		}
		else
			if(p[0] == 's') /* setter */
			{
				lgob_object_set(L);
				return 0;
			}
	}
	
	luaL_error(L, "attempt to call method '%s' (a nil value)", p);
	return 0;
}

static int __index(lua_State* L)
{
	lua_pushvalue(L, 2);
	lua_pushcclosure(L, index_closure, 1);
	
	return 1;
}

int luaopen_lgob_gobject(lua_State *L)
{
	/* Initialization */
	g_type_init();
	priv_handle_log(L, "GLib");
	priv_handle_log(L, "GThread");
	priv_handle_log(L, "GLib-GObject");
	
	luaL_loadstring(L, "require('lgob.common')"); lua_call(L, 0, 0);
	
	/* Register the classes */
	luaL_register(L, LIB_NAME, gobject);
	int top = lua_gettop(L);
	
	lua_pushliteral(L, "lgobObject");
	luaL_register(L, "gobject.Object", object);
	
	/* Set the object metatable */
	lua_newtable(L);
	lua_pushliteral(L, "__index");
	lua_pushcfunction(L, __index);
	lua_rawset(L, -3);
	
	lua_setmetatable(L, -2);
	
	lua_rawset(L, LUA_REGISTRYINDEX);
	luaL_register(L, "gobject.Type", type);
	
	/* Export some internal functions to other bindings */
	priv_register(L, "lgobObjectNew", lgob_object_new); 
	priv_register(L, "lgobStructNew", lgob_struct_new); 
	priv_register(L, "lgobBoxedNew", lgob_boxed_new); 
	priv_register(L, "lgobGc", lgob_object_gc); 
	priv_register(L, "lgobValuePush", lgob_value_push); 
	priv_register(L, "lgobValueSet", lgob_value_set); 
	priv_register(L, "lgobStructGc", lgob_struct_gc); 
	priv_register(L, "lgobStructTostring", lgob_struct_to_string); 
	priv_register(L, "lgobTostring", lgob_object_to_string); 
	priv_register(L, "lgobEq", lgob_object_eq);
	priv_register(L, "lgobClosureNew", lgob_closure_new);
	priv_register(L, "lgobGstrv2table", lgob_gstrv2table);
	priv_register(L, "lgobTable2gstrv", lgob_table2gstrv);
	priv_register(L, "lgobGSList2table", lgob_gslist2table);
	priv_register(L, "lgobGList2table", lgob_glist2table);
	
	/* Create a table to store class name exceptions */
	lua_newtable(L);
	lua_setfield(L, LUA_REGISTRYINDEX, "lgobSpecial");
	
	/* Create a table to store prefix */
	lua_newtable(L);
	lua_pushliteral(L, "G"); lua_pushliteral(L, "lgob"); lua_rawset(L, -3);
	lua_setfield(L, LUA_REGISTRYINDEX, "lgobPrefix");
	
	/* Register the GLib binding */
	luaL_register(L, "glib", glib);
	
	/* G_PRIORITY define */
	lua_pushliteral(L, "PRIORITY_HIGH"); lua_pushinteger(L, G_PRIORITY_HIGH); lua_rawset(L, -3);
	lua_pushliteral(L, "PRIORITY_DEFAULT"); lua_pushinteger(L, G_PRIORITY_DEFAULT); lua_rawset(L, -3);
	lua_pushliteral(L, "PRIORITY_HIGH_IDLE"); lua_pushinteger(L, G_PRIORITY_HIGH_IDLE); lua_rawset(L, -3);
	lua_pushliteral(L, "PRIORITY_DEFAULT_IDLE"); lua_pushinteger(L, G_PRIORITY_DEFAULT_IDLE); lua_rawset(L, -3);
	lua_pushliteral(L, "PRIORITY_LOW"); lua_pushinteger(L, G_PRIORITY_LOW); lua_rawset(L, -3);
	
	/* GNormalizeMode enum */
	lua_pushliteral(L, "NORMALIZE_DEFAULT"); lua_pushnumber(L, G_NORMALIZE_DEFAULT); lua_rawset(L, -3);
	lua_pushliteral(L, "NORMALIZE_NFD"); lua_pushnumber(L, G_NORMALIZE_NFD); lua_rawset(L, -3);
	lua_pushliteral(L, "NORMALIZE_DEFAULT_COMPOSE"); lua_pushnumber(L, G_NORMALIZE_DEFAULT_COMPOSE); lua_rawset(L, -3);
	lua_pushliteral(L, "NORMALIZE_NFC"); lua_pushnumber(L, G_NORMALIZE_NFC); lua_rawset(L, -3);
	lua_pushliteral(L, "NORMALIZE_ALL"); lua_pushnumber(L, G_NORMALIZE_ALL); lua_rawset(L, -3);
	lua_pushliteral(L, "NORMALIZE_NFKD"); lua_pushnumber(L, G_NORMALIZE_NFKD); lua_rawset(L, -3);
	lua_pushliteral(L, "NORMALIZE_ALL_COMPOSE"); lua_pushnumber(L, G_NORMALIZE_ALL_COMPOSE); lua_rawset(L, -3);
	lua_pushliteral(L, "NORMALIZE_NFKC"); lua_pushnumber(L, G_NORMALIZE_NFKC); lua_rawset(L, -3);
	
	luaL_register(L, "glib.Timeout", timeout);
	luaL_register(L, "glib.MainLoop", mainloop);
	luaL_register(L, "glib.Environment", environment);
	luaL_register(L, "glib.Path", path);
	luaL_register(L, "glib.Date", date);
	luaL_register(L, "glib.Rand", rand);
	luaL_register(L, "glib.TimeVal", timeval);
	luaL_register(L, "glib.Timer", timer);
	luaL_register(L, "glib.UTF8", utf8);
	
#if GLIB_MINOR_VERSION >= 12
	luaL_register(L, "glib.Base64", base64);
#endif

#if GLIB_MINOR_VERSION >= 16
	luaL_register(L, "glib.Checksum", checksum);
#endif
	
	if(luaL_newmetatable(L, "glibMainLoopMT"))
	{
		lua_getfield(L, LUA_REGISTRYINDEX, "lgobRegisterSpecial");
		lua_pushvalue(L, -2);
		lua_pushliteral(L, "glib");
		lua_pushliteral(L, "MainLoop");
		lua_pushcfunction(L, lgob_main_loop_free);
		lua_call(L, 4, 0);
	}
	
	if(luaL_newmetatable(L, "glibDateMT"))
	{
		lua_getfield(L, LUA_REGISTRYINDEX, "lgobRegisterSpecial");
		lua_pushvalue(L, -2);
		lua_pushliteral(L, "glib");
		lua_pushliteral(L, "Date");
		lua_pushcfunction(L, lgob_date_free);
		lua_call(L, 4, 0);
	}
	
	if(luaL_newmetatable(L, "glibRandMT"))
	{
		lua_getfield(L, LUA_REGISTRYINDEX, "lgobRegisterSpecial");
		lua_pushvalue(L, -2);
		lua_pushliteral(L, "glib");
		lua_pushliteral(L, "Rand");
		lua_pushcfunction(L, lgob_rand_free);
		lua_call(L, 4, 0);
	}
	
	if(luaL_newmetatable(L, "glibTimeValMT"))
	{
		lua_getfield(L, LUA_REGISTRYINDEX, "lgobRegisterSpecial");
		lua_pushvalue(L, -2);
		lua_pushliteral(L, "glib");
		lua_pushliteral(L, "TimeVal");
		lua_pushcfunction(L, lgob_time_val_free);
		lua_call(L, 4, 0);
	}
	
	if(luaL_newmetatable(L, "glibTimerMT"))
	{
		lua_getfield(L, LUA_REGISTRYINDEX, "lgobRegisterSpecial");
		lua_pushvalue(L, -2);
		lua_pushliteral(L, "glib");
		lua_pushliteral(L, "Timer");
		lua_pushcfunction(L, lgob_timer_free);
		lua_call(L, 4, 0);
	}
	
	/* lgob table must be in the top */
	lua_settop(L, top);

	return 1;
}
